﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using System.Collections;
using System.Security.Cryptography;
using System.Data.SqlClient;
using System.Data;
using System.Runtime.InteropServices;
using minsouk.WorkGenerator;

namespace minsouk.WorkGenerator
{
    class CKeyInput
    {
        InputEventArgs args;
        Dictionary<string, CThreadStatusDic> cThreadStatusDic;
        CThreadManager cThreadManager;
        CConfig config = null;
        object lockobj;
        bool isShowWorkThread;

        public CKeyInput(CThreadManager cThreadManager)
        {
            KeyInputEvent += cThreadManager.OnEventAction;
            this.lockobj = cThreadManager.GetLockObject();
            this.cThreadStatusDic = cThreadManager.GetCThreadStatusDic();
            this.isShowWorkThread = cThreadManager.GetShowWorkThreadStatus();
            this.cThreadManager = cThreadManager;
            this.config = cThreadManager.config;
        }

        public void DoDefaultWork()
        {
            List<WorkTypeCnt> list = config.GetWorkTypeCnt;

            foreach (var a in list )
            {
                for (var i = 0; i < Convert.ToInt32(a.cValue); i++)
                {
                    Console.WriteLine("{0}", a.workType);
                    DoWork(a.workType, "START");
                }
            }
        }

        private void StopWorkAll(int isExit)
        {
            List<Thread> threadList = new List<Thread>();
            Monitor.Enter(lockobj);
            try
            {
                var methodNameList = (from a in cThreadStatusDic
                                      orderby a.Value.threadNum descending
                                      select new { workMethodName = a.Value.workMethodName.ToString() }).ToList();

                foreach (var a in methodNameList)
                {
                    Thread thread = new Thread(() => DoWork(a.workMethodName, "STOP"));
                    threadList.Add(thread);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            finally
            {
                Monitor.Exit(lockobj);
            }

            foreach (var a in threadList)
            {
                a.Start();
            }

            foreach (var a in threadList)
            {
                a.Join();
            }
            if (isExit == 1)
            {
                Environment.Exit(0);
            }
        }

        private void DoWork(string methodName, string commnad)
        {
            args = new InputEventArgs();
            args.methodName = methodName;
            args.command = commnad;
            OnKeyInput(args);
        }

        private void ShowWorkThreadToggle()
        {
            args = new InputEventArgs();
            this.isShowWorkThread = cThreadManager.GetShowWorkThreadStatus();
            args.command = this.isShowWorkThread == true ? "DisableStatus" : "EnableStauts";
            OnKeyInput(args);
        }


        public void CommandPromptReadKey()
        {

            while (true)
            {
                if (Console.KeyAvailable)
                {
                    ConsoleKeyInfo cki;
                    cki = Console.ReadKey(true);
                    switch (cki.Key)
                    {
                        case ConsoleKey.D1:
                            DoWork("CWORK1", "START");
                            break;
                        case ConsoleKey.Q:
                            DoWork("CWORK1", "STOP");
                            break;

                        case ConsoleKey.D2:
                            DoWork("CWORK2", "START");
                            break;
                        case ConsoleKey.W:
                            DoWork("CWORK2", "STOP");
                            break;

                        case ConsoleKey.D3:
                            DoWork("CWORK3", "START");
                            break;
                        case ConsoleKey.E:
                            DoWork("CWORK3", "STOP");
                            break;

                        case ConsoleKey.S:
                            ShowWorkThreadToggle();
                            break;

                        case ConsoleKey.D9:
                            StopWorkAll(0); // stop work
                            break;

                        case ConsoleKey.D0:
                            StopWorkAll(1); // stop work and exit
                            break;

                        default:
                            Console.WriteLine("정의된 키가 아닙니다.");
                            break;
                    }
                }
                Thread.Sleep(50);
            }
        }

        // event execute function 
        protected virtual void OnKeyInput(InputEventArgs e)
        {
            EventHandler<InputEventArgs> handler = KeyInputEvent;
            if (handler != null)
            {
                handler(this, e);
            }
        }

        // event
        public event EventHandler<InputEventArgs> KeyInputEvent;
    }
}
