﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using System.Collections;
using System.Security.Cryptography;
using System.Data.SqlClient;
using System.Data;
using System.Runtime.InteropServices;
using minsouk.WorkGenerator;

namespace minsouk.WorkGenerator
{
    class CThreadManager
    {

        public CConfig config { get; set; }

        public CThreadManager(CConfig config)
        {
            this.config = config;
        }

        Dictionary<string, CThreadStatusDic> cThreadStatusDic = new Dictionary<string, CThreadStatusDic>();
        private object lockobj = new object();
        private int CWORK1Cnt, CWORK2Cnt, CWORK3Cnt;
        private bool isShowWorkThread = true;
        private string aliveShow = " ";

        public Dictionary<string, CThreadStatusDic> GetCThreadStatusDic()
        {
            return this.cThreadStatusDic;
        }

        public object GetLockObject()
        {
            return lockobj;
        }

        public bool GetShowWorkThreadStatus()
        {
            return isShowWorkThread;
        }
        // start
        private void StartWorkThread(string workMethodName)
        {
            Monitor.Enter(lockobj);
            try
            {
                int methodThreadNum =
                    (
                        from a in cThreadStatusDic
                        where a.Value.workMethodName == workMethodName
                        select a.Value.workMethodName
                    ).Count();
                
                // reflection thread start
                Type type = Type.GetType("minsouk.WorkGenerator." + workMethodName);  // class 이름이 들어있음 짜다보니....이렇게 되었음 
                object instance = Activator.CreateInstance(type, this);
                MethodInfo method = type.GetMethod("StartWork");
                PropertyInfo property = type.GetProperty("instanceId");
                property.SetValue(instance, methodThreadNum, null);
                Thread thread = new Thread(() => method.Invoke(instance, null));
                thread.Name = workMethodName + methodThreadNum;
                cThreadStatusDic.Add(workMethodName + methodThreadNum, new CThreadStatusDic(0, workMethodName, methodThreadNum, "", thread, (CWork)instance));
                thread.Start();

            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            finally
            {
                Monitor.Exit(lockobj);
            }
        }

        // stop
        private void StopWorkThread(string workMethodName)
        {
            Monitor.Enter(lockobj);
            try
            {
                if ((from a in cThreadStatusDic where a.Value.workMethodName == workMethodName select a.Value.workMethodName).Count() > 0)
                {
                    var Query = 
                        (from a in cThreadStatusDic
                         where a.Value.workMethodName == workMethodName
                         orderby a.Value.threadNum descending
                         select new { key = a.Key, iwork = a.Value.cWork }).Take(1);

                    foreach (var a in Query)
                    {
                        a.iwork.StopWork();
                        cThreadStatusDic.Remove(a.key);
                    }
                }
                else
                {
                    Console.WriteLine("해당 작업이 없습니다.");
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }
            finally
            {
                Monitor.Exit(lockobj);
            }
        }

        // update
        private void UpdateWorkThread(string methodName, int instanceId, string resultHashValue)
        {
            string key = methodName + instanceId;
            if (cThreadStatusDic.ContainsKey(key))
            {
                this.cThreadStatusDic[key].workCnt++;
                this.cThreadStatusDic[key].progressBar = this.cThreadStatusDic[key].progressBar.Length == 49 ? resultHashValue : this.cThreadStatusDic[key].progressBar + resultHashValue;
            }
        }

        // show 지저분함......
        public void ShowWorkThread()
        {
            int linqModeValue = 1;
            while (isShowWorkThread)
            {

                Monitor.Enter(this.lockobj);
                try
                {
                    this.CWORK1Cnt = (from a in cThreadStatusDic where a.Value.workMethodName == "CWORK1" select a).Count();
                    this.CWORK2Cnt = (from a in cThreadStatusDic where a.Value.workMethodName == "CWORK2" select a).Count();
                    this.CWORK3Cnt = (from a in cThreadStatusDic where a.Value.workMethodName == "CWORK3" select a).Count();

                    Console.Clear();
                    Console.SetWindowSize(75, Console.LargestWindowHeight);

                    if (this.CWORK1Cnt + this.CWORK2Cnt + this.CWORK3Cnt <= 30)
                    {
                        linqModeValue = 1;
                    }
                    else // thread 개수가 많다면, 
                    {
                        Console.WriteLine("Too many Lines !! this is Thread Summary ! ");
                        linqModeValue = 30;
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine("Linq Error ! Code : 1" + e.Message);
                }
                finally
                {
                    Monitor.Exit(lockobj);
                }

                Console.ForegroundColor = ConsoleColor.White;
                Console.Write("a.to.z@navercorp.com SQLStress TConfig.configIdx is : {0}", config.configIdx);
                Console.WriteLine(this.aliveShow = this.aliveShow.Length == 5 ? " ." : this.aliveShow + ".");
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("CMD [WorkLoad: 1,2,3 AND Q,W,E] [Show Status Toggle: S] [Exit: 0]");
                Console.ForegroundColor = ConsoleColor.White;
                Console.WriteLine("CWORK1Cnt ThreadCnt {0}, CWORK2Cnt ThreadCnt {1}, CWORK3Cnt ThreadCnt {2}", this.CWORK1Cnt, this.CWORK2Cnt, this.CWORK3Cnt);
                Console.WriteLine();
                Console.WriteLine("Type   No   Count      Progressing Status");
                Console.WriteLine("------ ---- ---------- -------------------------------------------------");

                Monitor.Enter(lockobj);
                try
                {
                    var Query =
                        from a in cThreadStatusDic
                        join b in
                            (
                            from a in cThreadStatusDic
                            group a by new { a.Value.workMethodName } into g
                            select new { methodName = g.Key.workMethodName, sumValue = g.Sum(x => x.Value.workCnt) }
                            )
                        on a.Value.workMethodName equals b.methodName
                        where a.Value.threadNum % linqModeValue == 0
                        orderby a.Value.workMethodName descending, a.Value.threadNum descending
                        select new
                        {
                            workMethodName = a.Value.workMethodName,
                            threadNum = a.Value.threadNum,
                            sumWorkCnt = b.sumValue,
                            workCnt = a.Value.workCnt,
                            progressBar = a.Value.progressBar
                        };


                    foreach (var threadStatus in Query)
                    {
                        if (threadStatus.workMethodName == "CWORK1")
                            Console.ForegroundColor = ConsoleColor.Magenta;
                        if (threadStatus.workMethodName == "CWORK2")
                            Console.ForegroundColor = ConsoleColor.Red;
                        if (threadStatus.workMethodName == "CWORK3")
                            Console.ForegroundColor = ConsoleColor.Cyan;
                        Console.WriteLine("{0} [{1,3}, {2,8}] {3}", threadStatus.workMethodName, threadStatus.threadNum, threadStatus.workCnt, threadStatus.progressBar);

                        if (threadStatus.threadNum == 0)
                        {
                            Console.ForegroundColor = ConsoleColor.Yellow;
                            Console.WriteLine("Thread Status of " + threadStatus.workMethodName + " ( Total Cnt : " + threadStatus.sumWorkCnt + " )");
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine("Linq Error ! Code : 2" + e.Message);
                }
                finally
                {
                    Monitor.Exit(lockobj);
                }

                Thread.Sleep(300);
            }
        }

        // OnEvent !!
        public void OnEventAction(object sender, InputEventArgs args)
        {
            if (sender is CKeyInput || sender is CConfig)
            {
                if (args.command == "START")
                {
                    StartWorkThread(args.methodName);
                }
                else if (args.command == "STOP")
                {
                    StopWorkThread(args.methodName);
                }
                else if (args.command == "EnableStauts")
                {
                    this.isShowWorkThread = true;
                    Thread workStatusShowThread = new Thread(() => ShowWorkThread());
                    workStatusShowThread.Start();
                }
                else if (args.command == "DisableStatus")
                {
                    this.isShowWorkThread = false;
                }
            }
            else if (sender is CWork)
            {
                UpdateWorkThread(args.methodName, args.instanceId, args.resultHashValue);
            }
        }

    }

    public class InputEventArgs : EventArgs
    {
        public string methodName { get; set; }
        public string resultHashValue { get; set; }
        public int instanceId { get; set; }
        public string command { get; set; }
    }

    class CThreadStatusDic
    {
        public int workCnt { get; set; }
        public string workMethodName { get; set; }
        public int threadNum { get; set; }
        public string progressBar { get; set; }
        public Thread thread { get; set; }
        public CWork cWork { get; set; }

        public CThreadStatusDic(int workCnt, string workMethodName, int threadNum, string progressBar, Thread thread, CWork cWork)
        {
            this.workCnt = workCnt;
            this.workMethodName = workMethodName;
            this.threadNum = threadNum;
            this.progressBar = progressBar;
            this.thread = thread;
            this.cWork = cWork;
        }
    }
}
